# -------------------------------------------------------------------------------------------------
# This script is to duplicate the seasonal verification done for Focus-Africa WP4.
# 
# Forecasting system: ECMWF SEAS5
# Lead times: 0-5 months
# Observation: ERA5
# Variables: prlr, tas
# Frequency: monthly (aggregated to 3-month seasonal sum/mean)
# Regrid: Both forecast and observation are interpolated to JMA-2 grid.
# Period: 1981-2016
# Calibration methods: RAW, bias, evmos and mse_min
# Skill metrics: corr, bss10, bss90, frpss
# Spatial domain: 0-60ºN, 40ºS-10ºN (the SADC region)
#
# Chung, June 2022
# -------------------------------------------------------------------------------------------------

# Output plots directory
output.dir <- '/esarchive/scratch/cchou/FOCUS/seasonal-verification/output/auto-s2s/plots/'
# Replace with the path to the recipe you want to use
recipe_file <- "modules/Loading/testing_recipes/c_recipe.yml"

source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")

# Load datasets
data <- load_datasets(recipe_file)

recipe <- read_yaml(recipe_file)
# Calibrate datasets
calibrated_data <- calibrate_datasets(data, recipe)
# Compute skill metrics
skill_metrics <- compute_skill_metrics(calibrated_data$hcst, data$obs,
                                       recipe, na.rm = T, ncores = 1)

# Take one lead time
corr_new <- skill_metrics$enscorr$enscorr_specs[, , , , 1, ,]
frpss_new <- skill_metrics$frpss$frpss_specs[, , , , 1, ,]
bss10_new <- skill_metrics$bss10[[1]][, , , , 1, ,]
bss90_new <- skill_metrics$bss90[[1]][, , , , 1, ,]

# Load the outputs from previous workflow
library(ncdf4)

system <- 'ecmwfs5'; calib_mode <- 'bias'; time_agg <- 'monthly'; var <- 'tas'; date <- '20210101'
userdirectory <- '/esarchive/scratch/cchou/FOCUS/seasonal-verification/output/JMAGrid/'
fname <- paste0(system, '/', calib_mode, '-', time_agg, '/', var, '/', date, '/')
nc_data <- nc_open(paste0(userdirectory, fname, var, '-skill_month', substr(date, 5, 6), '.nc'))

skill <- ncvar_get(nc_data, 'corr_r')
names(dim(skill)) <- c('lon', 'lat', 'leadtime')
corr_old <- skill[,,1]
skill <- ncvar_get(nc_data, 'frpss')
names(dim(skill)) <- c('lon', 'lat', 'leadtime')
frpss_old <- skill[,,1]
skill <- ncvar_get(nc_data, 'bss10')
names(dim(skill)) <- c('lon', 'lat', 'leadtime')
bss10_old <- skill[,,1]
skill <- ncvar_get(nc_data, 'bss90')
names(dim(skill)) <- c('lon', 'lat', 'leadtime')
bss90_old <- skill[,,1]





# Plot examples
fname <- paste0(var, '_', substr(date, 5, 6), '_corr_auto-s2s.png')
PlotEquiMap(corr_new,
            data$hcst$lon[1:length(data$hcst$lon)],
            data$hcst$lat[1:length(data$hcst$lat)],
            toptitle = "Correlation (auto-s2s)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))
fname <- paste0(var, '_', substr(date, 5, 6), '_corr_s2s4e.png')
PlotEquiMap(corr_old,
            data$hcst$lon[1:length(data$hcst$lon)],
            rev(data$hcst$lat[1:length(data$hcst$lat)]),
            toptitle = "Correlation (s2s4e)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))

fname <- paste0(var, '_', substr(date, 5, 6), '_frpss_auto-s2s.png')
PlotEquiMap(frpss_new,
            data$hcst$lon[1:length(data$hcst$lon)],
            data$hcst$lat[1:length(data$hcst$lat)],
            toptitle = "FRPSS (auto-s2s)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))
fname <- paste0(var, '_', substr(date, 5, 6), '_frpss_s2s4e.png')
PlotEquiMap(frpss_old,
            data$hcst$lon[1:length(data$hcst$lon)],
            rev(data$hcst$lat[1:length(data$hcst$lat)]),
            toptitle = "FRPSS (s2s4e)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))

fname <- paste0(var, '_', substr(date, 5, 6), '_bss10_auto-s2s.png')
PlotEquiMap(bss10_new,
            data$hcst$lon[1:length(data$hcst$lon)],
            data$hcst$lat[1:length(data$hcst$lat)],
            toptitle = "BSS10 (auto-s2s)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))
fname <- paste0(var, '_', substr(date, 5, 6), '_bss10_s2s4e.png')
PlotEquiMap(bss10_old,
            data$hcst$lon[1:length(data$hcst$lon)],
            rev(data$hcst$lat[1:length(data$hcst$lat)]),
            toptitle = "BSS10 (s2s4e)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))

fname <- paste0(var, '_', substr(date, 5, 6), '_bss90_auto-s2s.png')
PlotEquiMap(bss90_new,
            data$hcst$lon[1:length(data$hcst$lon)],
            data$hcst$lat[1:length(data$hcst$lat)],
            toptitle = "BSS90 (auto-s2s)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))
fname <- paste0(var, '_', substr(date, 5, 6), '_bss90_s2s4e.png')
PlotEquiMap(bss90_old,
            data$hcst$lon[1:length(data$hcst$lon)],
            rev(data$hcst$lat[1:length(data$hcst$lat)]),
            toptitle = "BSS90 (s2s4e)",
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(-1, 1, 0.1), width = 5, height = 6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, fname))



# -------------------------------------------------------------------------------------------------
# The script below takes advantage of the above data to test CSDownscale's interpolation function.
# 
# Chung, June 2022
# -------------------------------------------------------------------------------------------------

source('/esarchive/scratch/cchou/git/csdownscale/R/Interpolation.R')

output.dir <- '/esarchive/scratch/cchou/FOCUS/resolution/output/'

# ERA5 at SEAS5 grid
nc <- nc_open('/esarchive/scratch/cchou/FOCUS/seasonal-verification/output/SystemGrids/ecmwfs5/bias-monthly/tas/20210101/tas-obs_20160101_01.nc')
data <- ncvar_get(nc, 'tas')
names(dim(data)) <- c('lon', 'lat', 'leadtime')
era5 <- data[,,1]
era5.lon <- ncvar_get(nc, 'longitude')
era5.lat <- ncvar_get(nc, 'latitude')

fname <- paste0(var, '_', substr(date, 5, 6), '_era5.png')
PlotEquiMap(era5,
            era5.lon,
            era5.lat,
            toptitle = paste0('ERA5 ', var, '(201601)'),
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(14, 32, 2), width = 5, height = 6, title_scale = 0.6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, 'plots/', fname))



# ERA5 at JMA-2 grid

era5.jma2 <- drop(data$obs$data)
era5.jma2.lon <- data$obs$lon
era5.jma2.lat <- data$obs$lat
era5.jma2.2016 <- era5.jma2[36,1,,]-273.15

fname <- paste0(var, '_', substr(date, 5, 6), '_era5-jma2.png')
PlotEquiMap(era5.jma2.2016,
            era5.jma2.lon,
            era5.jma2.lat,
            toptitle = paste0('ERA5 ', var, ' at JMA-2 grid (201601)'),
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(14, 32, 2), width = 5, height = 6, title_scale = 0.6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, 'plots/', fname))

# Interpolate the above ERA5 data to r1440x721 grid with different methods

for (method in c('con', 'bil', 'bic', 'nn', 'con2')) {
print(paste0('Using the method ', method))

res <- Interpolation(exp = era5.jma2.2016,
lats = era5.jma2.lat,
lons = era5.jma2.lon,
method_remap = method,
# target_grid = 'r360x181',
target_grid = 'r3600x1801',
lat_dim = 'latitude',
lon_dim = 'longitude')

era5.jma2.era5 <- res$data
era5.jma2.era5.lon <- res$lon
era5.jma2.era5.lat <- res$lat

# Plot the before-after

#fname <- paste0(var, '_', substr(date, 5, 6), '_era5-jma2-era5_', method, '.png')
fname <- paste0(var, '_', substr(date, 5, 6), '_era5-jma2-era5land_', method, '.png')
PlotEquiMap(era5.jma2.era5,
            era5.jma2.era5.lon,
            era5.jma2.era5.lat,
#toptitle = paste0('JMA-2-grid ERA5 ', var, ' (201601)\n being interpolated to ERA5 grid (',
toptitle = paste0('JMA-2-grid ERA5 ', var, ' (201601)\n being interpolated to ERA5Land grid (',
method, ')'),
filled.continents = FALSE, filled.oceans = "lightgrey", country.borders = TRUE,
brks = seq(14, 32, 2), width = 5, height = 6, title_scale = 0.6,
triangle_ends_scale = 1, triangle_ends = c(FALSE,FALSE),
fileout = paste0(output.dir, 'plots/', fname))
rm(res, era5.jma2.era5)
}


